/*******************************************************************************
 * Copyright (c) 2014 Transcends, LLC.
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU 
 * General Public License as published by the Free Software Foundation; either version 2 of the 
 * License, or (at your option) any later version. This program is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You 
 * should have received a copy of the GNU General Public License along with this program; if not, 
 * write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, 
 * USA. 
 * http://www.gnu.org/licenses/gpl-2.0.html
 *******************************************************************************/
package org.rifidi.app.helloworld;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.rifidi.edge.api.AbstractRifidiApp;
import org.rifidi.edge.api.service.tagmonitor.ReadZone;
import org.rifidi.edge.api.service.tagmonitor.RSSIMonitoringService;
import org.rifidi.edge.api.service.tagmonitor.RSSIReadZoneSubscriber;

/**
 * A simple application which prints out arriving and departing tag for all readers.
 * 
 * @author Matthew Dean - matt@transcends.co
 */
public class HelloWorldApp extends AbstractRifidiApp {

	/** The service for monitoring arrival and departure events */
	private RSSIMonitoringService rssiMonitoringService;
	private List<RSSIReadZoneSubscriber> subscriberList;

	public HelloWorldApp(String group, String name) {
		super(group, name);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rifidi.edge.api.AbstractRifidiApp#_start()
	 */
	@Override
	public void _start() {
		super._start();

		HelloWorldSubscriber sub = new HelloWorldSubscriber();
		this.subscriberList = new LinkedList<RSSIReadZoneSubscriber>();
		this.subscriberList.add(sub);
		HashMap<String,ReadZone> readzones = new HashMap<String,ReadZone>();
		readzones.put("LLRP_1", new ReadZone("LLRP_1"));
		readzones.put("LLRP_2", new ReadZone("LLRP_2"));
		readzones.put("LLRP_3", new ReadZone("LLRP_3"));
		readzones.put("LLRP_4", new ReadZone("LLRP_4"));
		readzones.put("Alien_1", new ReadZone("Alien_1"));
		readzones.put("Alien_2", new ReadZone("Alien_2"));
		readzones.put("Alien_3", new ReadZone("Alien_3"));
		this.rssiMonitoringService.subscribe(sub, readzones, 4.0f, TimeUnit.SECONDS, 3, 2.0d, 0.0d, true);		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rifidi.edge.api.AbstractRifidiApp#_stop()
	 */
	@Override
	public void _stop() {
		for (RSSIReadZoneSubscriber s : this.subscriberList) {
			this.rssiMonitoringService.unsubscribe(s);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rifidi.edge.api.AbstractRifidiApp#initialize()
	 */
	@Override
	public void initialize() {
	}

	/**
	 * Called by spring. This method injects the ReadZoneMonitoringService into
	 * the application.
	 * 
	 * @param rzms
	 */
	public void setRssiMonitoringService(RSSIMonitoringService rzms) {
		this.rssiMonitoringService = rzms;
	}
}
